@@ -39,12 +39,12 @@
 #pragma once
 #define I_IOBuffer_h
 
-#include "ts/ink_platform.h"
-#include "ts/ink_apidefs.h"
-#include "ts/Allocator.h"
-#include "ts/Ptr.h"
-#include "ts/ink_assert.h"
-#include "ts/ink_resource.h"
+#include "tscore/ink_platform.h"
+#include "tscore/ink_apidefs.h"
+#include "tscore/Allocator.h"
+#include "tscore/Ptr.h"
+#include "tscore/ink_assert.h"
+#include "tscore/ink_resource.h"
 
 struct MIOBufferAccessor;
 
@@ -65,11 +65,8 @@ extern int64_t default_large_iobuffer_size; // matched to size of OS buffers
 
 enum AllocType {
   NO_ALLOC,
-  FAST_ALLOCATED,
-  XMALLOCED,
   MEMALIGNED,
   DEFAULT_ALLOC,
-  CONSTANT,
 };
 
 #define DEFAULT_BUFFER_NUMBER 128
@@ -146,14 +143,6 @@ void init_buffer_allocators(int iobuffer_advice);
       <td>NO_ALLOC</td>
       <td></td>
     </tr>
-    <tr>
-      <td>FAST_ALLOCATED</td>
-      <td></td>
-    </tr>
-    <tr>
-      <td>XMALLOCED</td>
-      <td></td>
-    </tr>
     <tr>
       <td>MEMALIGNED</td>
       <td></td>
@@ -162,10 +151,6 @@ void init_buffer_allocators(int iobuffer_advice);
       <td>DEFAULT_ALLOC</td>
       <td></td>
     </tr>
-    <tr>
-      <td>CONSTANT</td>
-      <td></td>
-    </tr>
   </table>
 
  */
@@ -228,7 +213,7 @@ public:
     should not use this object or reference after this call.
 
   */
-  virtual void free();
+  void free() override;
 
   int64_t _size_index;
 
@@ -269,10 +254,9 @@ public:
   {
   }
 
-private:
-  // declaration only
-  IOBufferData(const IOBufferData &);
-  IOBufferData &operator=(const IOBufferData &);
+  // noncopyable, declaration only
+  IOBufferData(const IOBufferData &) = delete;
+  IOBufferData &operator=(const IOBufferData &) = delete;
 };
 
 inkcoreapi extern ClassAllocator<IOBufferData> ioDataAllocator;
@@ -478,8 +462,6 @@ public:
   void realloc_set_internal(void *b, int64_t buf_size, int64_t asize_index);
   void realloc(void *b, int64_t buf_size);
   void realloc(int64_t i);
-  void realloc_xmalloc(void *b, int64_t buf_size);
-  void realloc_xmalloc(int64_t buf_size);
 
   /**
     Frees the IOBufferBlock object and its underlying memory.
@@ -488,7 +470,7 @@ public:
     call.
 
   */
-  virtual void free();
+  void free() override;
 
   char *_start;
   char *_end;
@@ -520,9 +502,9 @@ public:
   */
   IOBufferBlock();
 
-private:
-  IOBufferBlock(const IOBufferBlock &);
-  IOBufferBlock &operator=(const IOBufferBlock &);
+  // noncopyable
+  IOBufferBlock(const IOBufferBlock &) = delete;
+  IOBufferBlock &operator=(const IOBufferBlock &) = delete;
 };
 
 extern inkcoreapi ClassAllocator<IOBufferBlock> ioBlockAllocator;
@@ -840,13 +822,16 @@ public:
   void append_block(int64_t asize_index);
 
   /**
-    Adds new block to the end of block list using the block size for
-    the buffer specified when the buffer was allocated.
-
+    Adds a new block to the end of the block list. Note that this does nothing when the next block of the current writer exists.
+    The block size is the same as specified size when the buffer was allocated.
   */
   void add_block();
 
   /**
+    Deprecated
+
+    TODO: remove this function. Because ats_xmalloc() doesn't exist anymore.
+
     Adds by reference len bytes of data pointed to by b to the end
     of the buffer.  b MUST be a pointer to the beginning of  block
     allocated from the ats_xmalloc() routine. The data will be deallocated
@@ -936,7 +921,7 @@ public:
   buf()
   {
     IOBufferBlock *b = first_write_block();
-    return b ? b->buf() : 0;
+    return b ? b->buf() : nullptr;
   }
 
   char *
@@ -1005,7 +990,7 @@ public:
   bool
   high_water()
   {
-    return max_read_avail() > water_mark;
+    return is_max_read_avail_more_than(this->water_mark);
   }
 
   /**
@@ -1075,9 +1060,7 @@ public:
   void dealloc_all_readers();
 
   void set(void *b, int64_t len);
-  void set_xmalloced(void *b, int64_t len);
   void alloc(int64_t i = default_large_iobuffer_size);
-  void alloc_xmalloc(int64_t buf_size);
   void append_block_internal(IOBufferBlock *b);
   int64_t puts(char *buf, int64_t len);
 
@@ -1088,8 +1071,25 @@ public:
   {
     return !_writer;
   }
+
+  /**
+    Get the maximum amount of available data across all of the readers.
+    If there're no allocated reader, return available data size of current writer.
+
+    This calls IOBufferReader::read_avail() and it could be expensive when it has a ton of IOBufferBlock.
+    The `is_max_read_avail(int64_t size)` is preferred if possible.
+
+    @return maximum amount of available data
+   */
   int64_t max_read_avail();
 
+  /**
+    Check if there is more than @a size bytes available to read.
+
+    @return @c true if more than @a size byte are available.
+  */
+  bool is_max_read_avail_more_than(int64_t size);
+
   int max_block_count();
   void check_add_block();
 
@@ -1101,18 +1101,21 @@ public:
     if (_writer) {
       _writer->reset();
     }
-    for (int j = 0; j < MAX_MIOBUFFER_READERS; j++)
-      if (readers[j].allocated()) {
-        readers[j].reset();
+    for (auto &reader : readers) {
+      if (reader.allocated()) {
+        reader.reset();
       }
+    }
   }
 
   void
   init_readers()
   {
-    for (int j = 0; j < MAX_MIOBUFFER_READERS; j++)
-      if (readers[j].allocated() && !readers[j].block)
-        readers[j].block = _writer;
+    for (auto &reader : readers) {
+      if (reader.allocated() && !reader.block) {
+        reader.block = _writer;
+      }
+    }
   }
 
   void
@@ -1140,16 +1143,6 @@ public:
   {
     _writer->realloc(b, buf_size);
   }
-  void
-  realloc_xmalloc(void *b, int64_t buf_size)
-  {
-    _writer->realloc_xmalloc(b, buf_size);
-  }
-  void
-  realloc_xmalloc(int64_t buf_size)
-  {
-    _writer->realloc_xmalloc(buf_size);
-  }
 
   int64_t size_index;
 
@@ -1182,13 +1175,13 @@ public:
 */
 struct MIOBufferAccessor {
   IOBufferReader *
-  reader()
+  reader() const
   {
     return entry;
   }
 
   MIOBuffer *
-  writer()
+  writer() const
   {
     return mbuf;
   }
@@ -1232,10 +1225,11 @@ struct MIOBufferAccessor {
   const char *name;
 #endif
 
-private:
-  MIOBufferAccessor(const MIOBufferAccessor &);
-  MIOBufferAccessor &operator=(const MIOBufferAccessor &);
+  // noncopyable
+  MIOBufferAccessor(const MIOBufferAccessor &) = delete;
+  MIOBufferAccessor &operator=(const MIOBufferAccessor &) = delete;
 
+private:
   MIOBuffer *mbuf;
   IOBufferReader *entry;
 };
@@ -1297,7 +1291,7 @@ extern IOBufferBlock *new_IOBufferBlock_internal(
 #ifdef TRACK_BUFFER_USER
   const char *loc
 #endif
-  );
+);
 
 extern IOBufferBlock *new_IOBufferBlock_internal(
 #ifdef TRACK_BUFFER_USER
@@ -1345,12 +1339,6 @@ extern IOBufferData *new_xmalloc_IOBufferData_internal(
 #endif
   void *b, int64_t size);
 
-extern IOBufferData *new_constant_IOBufferData_internal(
-#ifdef TRACK_BUFFER_USER
-  const char *locaction,
-#endif
-  void *b, int64_t size);
-
 #ifdef TRACK_BUFFER_USER
 class IOBufferData_tracker
 {
@@ -1366,14 +1354,13 @@ public:
 };
 #endif
 
+// TODO: remove new_xmalloc_IOBufferData. Because ats_xmalloc() doesn't exist anymore.
 #ifdef TRACK_BUFFER_USER
 #define new_IOBufferData IOBufferData_tracker(RES_PATH("memory/IOBuffer/"))
 #define new_xmalloc_IOBufferData(b, size) new_xmalloc_IOBufferData_internal(RES_PATH("memory/IOBuffer/"), (b), (size))
-#define new_constant_IOBufferData(b, size) new_constant_IOBufferData_internal(RES_PATH("memory/IOBuffer/"), (b), (size))
 #else
 #define new_IOBufferData new_IOBufferData_internal
 #define new_xmalloc_IOBufferData new_xmalloc_IOBufferData_internal
-#define new_constant_IOBufferData new_constant_IOBufferData_internal
 #endif
 
 extern int64_t iobuffer_size_to_index(int64_t size, int64_t max = max_iobuffer_size);
