@@ -34,44 +34,22 @@
  ****************************************************************************/
 #pragma once
 
-#include "ts/ink_platform.h"
+#include "tscore/ink_platform.h"
 #include "I_Event.h"
 struct ProtectedQueue {
   void enqueue(Event *e, bool fast_signal = false);
   void signal();
-  int try_signal(); // Use non blocking lock and if acquired, signal
+  int try_signal();             // Use non blocking lock and if acquired, signal
+  void enqueue_local(Event *e); // Safe when called from the same thread
   void remove(Event *e);
-
-  /// Add an event to the thread local queue.
-  /// @note Must be called from the owner thread.
-  void enqueue_local(Event *e);
-
-  /// Get an event from the thread local queue.
-  /// @note Must be called from the owner thread.
   Event *dequeue_local();
-
-  /// Attempt to dequeue, waiting for @a timeout if there's no data.
   void dequeue_timed(ink_hrtime cur_time, ink_hrtime timeout, bool sleep);
+  void dequeue_external();       // Dequeue any external events.
+  void wait(ink_hrtime timeout); // Wait for @a timeout nanoseconds on a condition variable if there are no events.
 
-  /// Dequeue any external events.
-  void dequeue_external();
-
-  /// Wait for @a timeout nanoseconds on a condition variable if there are no events.
-  void wait(ink_hrtime timeout);
-  /// Events added from other threads.
   InkAtomicList al;
-
-  /// Lock for condition variable
-  /// pthread_cond_wait requires we lock this mutex before calling and it's used for
-  /// the signal logic to avoid race conditions.
   ink_mutex lock;
-
-  /// Condition variable for timed wait.
   ink_cond might_have_data;
-
-  /** This is a queue for events scheduled from the same thread. The @c _local methods use this
-      queue and should never be called from another thread.
-  */
   Que(Event, link) localQueue;
 
   ProtectedQueue();
