@@ -32,14 +32,13 @@
  *
  */
 
-#ifndef _ALARMS_H
-#define _ALARMS_H
+#pragma once
 
-#include <stdlib.h>
-#include <stdio.h>
+#include <cstdlib>
+#include <cstdio>
 
-#include "ts/ink_hash_table.h"
-#include "ts/ink_mutex.h"
+#include "tscore/ink_hash_table.h"
+#include "tscore/ink_mutex.h"
 
 class AppVersionInfo;
 
@@ -70,18 +69,14 @@ class AppVersionInfo;
 #define MGMT_ALARM_WEB_ERROR 15
 #define MGMT_ALARM_PING_FAILURE 16
 #define MGMT_ALARM_MGMT_CONFIG_ERROR 17
-#define MGMT_ALARM_ADD_ALARM 18                    /* OEM_ALARM */
-#define MGMT_ALARM_PROXY_LOG_SPACE_ROLLED 19       /* Alarm when log files will be rolled */
-#define MGMT_ALARM_PROXY_HTTP_CONGESTED_SERVER 20  /* Congestion control -- congested server */
-#define MGMT_ALARM_PROXY_HTTP_ALLEVIATED_SERVER 21 /* Congestion control -- alleviated server */
-#define MGMT_ALARM_PROXY_FTP_ERROR 22
+#define MGMT_ALARM_ADD_ALARM 18              /* OEM_ALARM */
+#define MGMT_ALARM_PROXY_LOG_SPACE_ROLLED 19 /* Alarm when log files will be rolled */
 
 #define MGMT_ALARM_SAC_SERVER_DOWN 400
 
 extern const char *alarmText[];
 extern const int alarmTextNum;
 
-
 /* OEM_ALARM: the alarm type is used as a key for hash tables;
    need offset and modulo constants which will keep the unique
    keys for OEM alarms within a specified range */
@@ -108,10 +103,10 @@ public:
   ~Alarms();
 
   void registerCallback(AlarmCallbackFunc func);
-  bool isCurrentAlarm(alarm_t a, char *ip = NULL);
+  bool isCurrentAlarm(alarm_t a, char *ip = nullptr);
 
-  void signalAlarm(alarm_t t, const char *desc, const char *ip = NULL);
-  void resolveAlarm(alarm_t a, char *ip = NULL);
+  void signalAlarm(alarm_t t, const char *desc, const char *ip = nullptr);
+  void resolveAlarm(alarm_t a, char *ip = nullptr);
 
   void constructAlarmMessage(const AppVersionInfo &version, char *ip, char *message, int max);
   void resetSeenFlag(char *ip);
@@ -141,5 +136,3 @@ private:
   int alarmOEMcount;
 
 }; /* End class Alarms */
-
-#endif /* _ALARMS_H */
