@@ -22,14 +22,28 @@
 #include <sys/time.h>
 #include <unistd.h>
 #include <arpa/inet.h>
-#include <ctype.h>
+#include <cctype>
 #include <sstream>
+#include <array>
+#include <atomic>
 
 #include "ts/ts.h"
 
 #include "conditions.h"
+#include "expander.h"
 #include "lulu.h"
 
+// This is a bit of a hack, to get the more linux specific tcp_info struct ...
+#if HAVE_STRUCT_LINUX_TCP_INFO
+#ifndef _LINUX_TCP_H
+#define _LINUX_TCP_H
+#endif
+#elif HAVE_NETINET_IN_H
+#ifndef _NETINET_TCP_H
+#define _NETINET_TCP_H
+#endif
+#endif
+
 // ConditionStatus
 void
 ConditionStatus::initialize(Parser &p)
@@ -37,7 +51,7 @@ ConditionStatus::initialize(Parser &p)
   Condition::initialize(p);
   MatcherType *match = new MatcherType(_cond_op);
 
-  match->set(static_cast<TSHttpStatus>(strtol(p.get_arg().c_str(), NULL, 10)));
+  match->set(static_cast<TSHttpStatus>(strtol(p.get_arg().c_str(), nullptr, 10)));
   _matcher = match;
 
   require_resources(RSRC_SERVER_RESPONSE_HEADERS);
@@ -62,10 +76,7 @@ ConditionStatus::eval(const Resources &res)
 void
 ConditionStatus::append_value(std::string &s, const Resources &res)
 {
-  std::ostringstream oss;
-
-  oss << res.resp_status;
-  s += oss.str();
+  s += std::to_string(res.resp_status);
   TSDebug(PLUGIN_NAME, "Appending STATUS(%d) to evaluation value -> %s", res.resp_status, s.c_str());
 }
 
@@ -117,11 +128,11 @@ ConditionRandom::initialize(Parser &p)
   Condition::initialize(p);
   MatcherType *match = new MatcherType(_cond_op);
 
-  gettimeofday(&tv, NULL);
+  gettimeofday(&tv, nullptr);
   _seed = getpid() * tv.tv_usec;
-  _max  = strtol(_qualifier.c_str(), NULL, 10);
+  _max  = strtol(_qualifier.c_str(), nullptr, 10);
 
-  match->set(static_cast<unsigned int>(strtol(p.get_arg().c_str(), NULL, 10)));
+  match->set(static_cast<unsigned int>(strtol(p.get_arg().c_str(), nullptr, 10)));
   _matcher = match;
 }
 
@@ -149,7 +160,7 @@ ConditionAccess::initialize(Parser &p)
   struct timeval tv;
   Condition::initialize(p);
 
-  gettimeofday(&tv, NULL);
+  gettimeofday(&tv, nullptr);
 
   _next = tv.tv_sec + 2;
   _last = !access(_qualifier.c_str(), R_OK);
@@ -170,14 +181,14 @@ ConditionAccess::eval(const Resources & /* res ATS_UNUSED */)
 {
   struct timeval tv;
 
-  gettimeofday(&tv, NULL);
+  gettimeofday(&tv, nullptr);
   if (tv.tv_sec > _next) {
     // There is a small "race" here, where we could end up calling access() a few times extra. I think
     // that is OK, and not worth protecting with a lock.
     bool check = !access(_qualifier.c_str(), R_OK);
 
     tv.tv_sec += 2;
-    mb();
+    std::atomic_thread_fence(std::memory_order_seq_cst);
     _next = tv.tv_sec; // I hope this is an atomic "set"...
     _last = check;     // This sure ought to be
   }
@@ -231,7 +242,7 @@ ConditionHeader::append_value(std::string &s, const Resources &res)
       s.append(value, len);
       // multiple headers with the same name must be semantically the same as one value which is comma separated
       if (next_field_loc) {
-        s.append(",");
+        s += ',';
       }
       TSHandleMLocRelease(bufp, hdr_loc, field_loc);
       field_loc = next_field_loc;
@@ -314,7 +325,7 @@ ConditionQuery::append_value(std::string &s, const Resources &res)
 bool
 ConditionQuery::eval(const Resources &res)
 {
-  if (NULL != res._rri) {
+  if (nullptr != res._rri) {
     std::string s;
 
     append_value(s, res);
@@ -348,19 +359,12 @@ ConditionUrl::set_qualifier(const std::string &q)
 }
 
 void
-ConditionUrl::append_value(std::string & /* s ATS_UNUSED */, const Resources & /* res ATS_UNUSED */)
+ConditionUrl::append_value(std::string &s, const Resources &res)
 {
-}
+  TSMLoc url     = nullptr;
+  TSMBuffer bufp = nullptr;
 
-bool
-ConditionUrl::eval(const Resources &res)
-{
-  TSDebug(PLUGIN_NAME, "ConditionUrl::eval");
-  TSMLoc url     = NULL;
-  TSMBuffer bufp = NULL;
-  std::string s;
-
-  if (res._rri != NULL) {
+  if (res._rri != nullptr) {
     // called at the remap hook
     bufp = res._rri->requestBufp;
     if (_type == URL || _type == CLIENT) {
@@ -375,10 +379,10 @@ ConditionUrl::eval(const Resources &res)
       url = res._rri->mapToUrl;
     } else {
       TSError("[header_rewrite] Invalid option value");
-      return false;
+      return;
     }
   } else {
-    TSMLoc hdr_loc = NULL;
+    TSMLoc hdr_loc = nullptr;
     if (_type == CLIENT) {
       bufp    = res.client_bufp;
       hdr_loc = res.client_hdr_loc;
@@ -387,21 +391,66 @@ ConditionUrl::eval(const Resources &res)
       hdr_loc = res.hdr_loc;
     } else {
       TSError("[header_rewrite] Rule not supported at this hook");
-      return false;
+      return;
     }
     if (TSHttpHdrUrlGet(bufp, hdr_loc, &url) != TS_SUCCESS) {
       TSError("[header_rewrite] Error getting the URL");
-      return false;
+      return;
     }
   }
 
-  if (_url_qual == URL_QUAL_HOST) {
-    int host_len     = 0;
-    const char *host = TSUrlHostGet(bufp, url, &host_len);
-    s.append(host, host_len);
-    TSDebug(PLUGIN_NAME, "   Host to match is: %.*s", host_len, host);
-  }
+  int i;
+  const char *q_str;
 
+  switch (_url_qual) {
+  case URL_QUAL_HOST:
+    q_str = TSUrlHostGet(bufp, url, &i);
+    s.append(q_str, i);
+    TSDebug(PLUGIN_NAME, "   Host to match is: %.*s", i, q_str);
+    break;
+  case URL_QUAL_PORT:
+    i = TSUrlPortGet(bufp, url);
+    s.append(std::to_string(i));
+    TSDebug(PLUGIN_NAME, "   Port to match is: %d", i);
+    break;
+  case URL_QUAL_PATH:
+    q_str = TSUrlPathGet(bufp, url, &i);
+    s.append(q_str, i);
+    TSDebug(PLUGIN_NAME, "   Path to match is: %.*s", i, q_str);
+    break;
+  case URL_QUAL_QUERY:
+    q_str = TSUrlHttpQueryGet(bufp, url, &i);
+    s.append(q_str, i);
+    TSDebug(PLUGIN_NAME, "   Query parameters to match is: %.*s", i, q_str);
+    break;
+  case URL_QUAL_MATRIX:
+    q_str = TSUrlHttpParamsGet(bufp, url, &i);
+    s.append(q_str, i);
+    TSDebug(PLUGIN_NAME, "   Matrix parameters to match is: %.*s", i, q_str);
+    break;
+  case URL_QUAL_SCHEME:
+    q_str = TSUrlSchemeGet(bufp, url, &i);
+    s.append(q_str, i);
+    TSDebug(PLUGIN_NAME, "   Scheme to match is: %.*s", i, q_str);
+    break;
+  case URL_QUAL_URL:
+  case URL_QUAL_NONE: {
+    // TSUrlStringGet returns an allocated char * we must free
+    char *non_const_q_str = TSUrlStringGet(bufp, url, &i);
+    s.append(non_const_q_str, i);
+    TSDebug(PLUGIN_NAME, "   URL to match is: %.*s", i, non_const_q_str);
+    TSfree(non_const_q_str);
+    break;
+  }
+  }
+}
+
+bool
+ConditionUrl::eval(const Resources &res)
+{
+  std::string s;
+
+  append_value(s, res);
   return static_cast<const Matchers<std::string> *>(_matcher)->test(s);
 }
 
@@ -498,19 +547,19 @@ ConditionCookie::append_value(std::string &s, const Resources &res)
   const int cookie_name_len     = _qualifier.length();
 
   // Sanity
-  if (bufp == NULL || hdr_loc == NULL) {
+  if (bufp == nullptr || hdr_loc == nullptr) {
     return;
   }
 
   // Find Cookie
   field_loc = TSMimeHdrFieldFind(bufp, hdr_loc, TS_MIME_FIELD_COOKIE, TS_MIME_LEN_COOKIE);
-  if (field_loc == NULL) {
+  if (field_loc == nullptr) {
     return;
   }
 
   // Get all cookies
   cookies = TSMimeHdrFieldValueStringGet(bufp, hdr_loc, field_loc, -1, &cookies_len);
-  if (cookies == NULL || cookies_len <= 0) {
+  if (cookies == nullptr || cookies_len <= 0) {
     goto out_release_field;
   }
 
@@ -550,7 +599,7 @@ ConditionInternalTxn::eval(const Resources &res)
 }
 
 void
-ConditionClientIp::initialize(Parser &p)
+ConditionIp::initialize(Parser &p)
 {
   Condition::initialize(p);
 
@@ -560,24 +609,62 @@ ConditionClientIp::initialize(Parser &p)
   _matcher = match;
 }
 
+void
+ConditionIp::set_qualifier(const std::string &q)
+{
+  Condition::set_qualifier(q);
+
+  TSDebug(PLUGIN_NAME, "\tParsing %%{IP:%s} qualifier", q.c_str());
+
+  if (q == "CLIENT") {
+    _ip_qual = IP_QUAL_CLIENT;
+  } else if (q == "INBOUND") {
+    _ip_qual = IP_QUAL_INBOUND;
+  } else if (q == "SERVER") {
+    _ip_qual = IP_QUAL_SERVER;
+  } else if (q == "OUTBOUND") {
+    _ip_qual = IP_QUAL_OUTBOUND;
+  } else {
+    TSError("[%s] Unknown IP() qualifier: %s", PLUGIN_NAME, q.c_str());
+  }
+}
+
 bool
-ConditionClientIp::eval(const Resources &res)
+ConditionIp::eval(const Resources &res)
 {
   std::string s;
 
   append_value(s, res);
-  TSDebug(PLUGIN_NAME, "Evaluating CLIENT-IP()");
+  bool rval = static_cast<const Matchers<std::string> *>(_matcher)->test(s);
 
-  return static_cast<MatcherType *>(_matcher)->test(s);
+  TSDebug(PLUGIN_NAME, "Evaluating IP(): %s - rval: %d", s.c_str(), rval);
+
+  return rval;
 }
 
 void
-ConditionClientIp::append_value(std::string &s, const Resources &res)
+ConditionIp::append_value(std::string &s, const Resources &res)
 {
+  bool ip_set = false;
   char ip[INET6_ADDRSTRLEN];
 
-  if (getIP(TSHttpTxnClientAddrGet(res.txnp), ip)) {
-    s.append(ip);
+  switch (_ip_qual) {
+  case IP_QUAL_CLIENT:
+    ip_set = (nullptr != getIP(TSHttpTxnClientAddrGet(res.txnp), ip));
+    break;
+  case IP_QUAL_INBOUND:
+    ip_set = (nullptr != getIP(TSHttpTxnIncomingAddrGet(res.txnp), ip));
+    break;
+  case IP_QUAL_SERVER:
+    ip_set = (nullptr != getIP(TSHttpTxnServerAddrGet(res.txnp), ip));
+    break;
+  case IP_QUAL_OUTBOUND:
+    ip_set = (nullptr != getIP(TSHttpTxnOutgoingAddrGet(res.txnp), ip));
+    break;
+  }
+
+  if (ip_set) {
+    s += ip;
   }
 }
 
@@ -588,7 +675,7 @@ ConditionIncomingPort::initialize(Parser &p)
 
   MatcherType *match = new MatcherType(_cond_op);
 
-  match->set(static_cast<uint16_t>(strtoul(p.get_arg().c_str(), NULL, 10)));
+  match->set(static_cast<uint16_t>(strtoul(p.get_arg().c_str(), nullptr, 10)));
   _matcher = match;
 }
 
@@ -620,7 +707,7 @@ ConditionTransactCount::initialize(Parser &p)
   MatcherType *match     = new MatcherType(_cond_op);
   std::string const &arg = p.get_arg();
 
-  match->set(strtol(arg.c_str(), NULL, 10));
+  match->set(strtol(arg.c_str(), nullptr, 10));
   _matcher = match;
 }
 
@@ -710,7 +797,7 @@ ConditionNow::initialize(Parser &p)
 
   MatcherType *match = new MatcherType(_cond_op);
 
-  match->set(static_cast<int64_t>(strtol(p.get_arg().c_str(), NULL, 10)));
+  match->set(static_cast<int64_t>(strtol(p.get_arg().c_str(), nullptr, 10)));
   _matcher = match;
 }
 
@@ -766,60 +853,62 @@ ConditionNow::eval(const Resources &res)
 const char *
 ConditionGeo::get_geo_string(const sockaddr *addr) const
 {
-  const char *ret = NULL;
+  const char *ret = "(unknown)";
   int v           = 4;
 
-  switch (_geo_qual) {
-  // Country database
-  case GEO_QUAL_COUNTRY:
-    switch (addr->sa_family) {
-    case AF_INET:
-      if (gGeoIP[GEOIP_COUNTRY_EDITION]) {
-        uint32_t ip = ntohl(reinterpret_cast<const struct sockaddr_in *>(addr)->sin_addr.s_addr);
+  if (addr) {
+    switch (_geo_qual) {
+    // Country database
+    case GEO_QUAL_COUNTRY:
+      switch (addr->sa_family) {
+      case AF_INET:
+        if (gGeoIP[GEOIP_COUNTRY_EDITION]) {
+          uint32_t ip = ntohl(reinterpret_cast<const struct sockaddr_in *>(addr)->sin_addr.s_addr);
 
-        ret = GeoIP_country_code_by_ipnum(gGeoIP[GEOIP_COUNTRY_EDITION], ip);
+          ret = GeoIP_country_code_by_ipnum(gGeoIP[GEOIP_COUNTRY_EDITION], ip);
+        }
+        break;
+      case AF_INET6: {
+        if (gGeoIP[GEOIP_COUNTRY_EDITION_V6]) {
+          geoipv6_t ip = reinterpret_cast<const struct sockaddr_in6 *>(addr)->sin6_addr;
+
+          v   = 6;
+          ret = GeoIP_country_code_by_ipnum_v6(gGeoIP[GEOIP_COUNTRY_EDITION_V6], ip);
+        }
+      } break;
+      default:
+        break;
       }
+      TSDebug(PLUGIN_NAME, "eval(): Client IPv%d seems to come from Country: %s", v, ret);
       break;
-    case AF_INET6: {
-      if (gGeoIP[GEOIP_COUNTRY_EDITION_V6]) {
-        geoipv6_t ip = reinterpret_cast<const struct sockaddr_in6 *>(addr)->sin6_addr;
 
-        v   = 6;
-        ret = GeoIP_country_code_by_ipnum_v6(gGeoIP[GEOIP_COUNTRY_EDITION_V6], ip);
+    // ASN database
+    case GEO_QUAL_ASN_NAME:
+      switch (addr->sa_family) {
+      case AF_INET:
+        if (gGeoIP[GEOIP_ASNUM_EDITION]) {
+          uint32_t ip = ntohl(reinterpret_cast<const struct sockaddr_in *>(addr)->sin_addr.s_addr);
+
+          ret = GeoIP_name_by_ipnum(gGeoIP[GEOIP_ASNUM_EDITION], ip);
+        }
+        break;
+      case AF_INET6: {
+        if (gGeoIP[GEOIP_ASNUM_EDITION_V6]) {
+          geoipv6_t ip = reinterpret_cast<const struct sockaddr_in6 *>(addr)->sin6_addr;
+
+          v   = 6;
+          ret = GeoIP_name_by_ipnum_v6(gGeoIP[GEOIP_ASNUM_EDITION_V6], ip);
+        }
+      } break;
+      default:
+        break;
       }
-    } break;
+      TSDebug(PLUGIN_NAME, "eval(): Client IPv%d seems to come from ASN Name: %s", v, ret);
+      break;
+
     default:
       break;
     }
-    TSDebug(PLUGIN_NAME, "eval(): Client IPv%d seems to come from Country: %s", v, ret);
-    break;
-
-  // ASN database
-  case GEO_QUAL_ASN_NAME:
-    switch (addr->sa_family) {
-    case AF_INET:
-      if (gGeoIP[GEOIP_ASNUM_EDITION]) {
-        uint32_t ip = ntohl(reinterpret_cast<const struct sockaddr_in *>(addr)->sin_addr.s_addr);
-
-        ret = GeoIP_name_by_ipnum(gGeoIP[GEOIP_ASNUM_EDITION], ip);
-      }
-      break;
-    case AF_INET6: {
-      if (gGeoIP[GEOIP_ASNUM_EDITION_V6]) {
-        geoipv6_t ip = reinterpret_cast<const struct sockaddr_in6 *>(addr)->sin6_addr;
-
-        v   = 6;
-        ret = GeoIP_name_by_ipnum_v6(gGeoIP[GEOIP_ASNUM_EDITION_V6], ip);
-      }
-    } break;
-    default:
-      break;
-    }
-    TSDebug(PLUGIN_NAME, "eval(): Client IPv%d seems to come from ASN Name: %s", v, ret);
-    break;
-
-  default:
-    break;
   }
 
   return ret ? ret : "(unknown)";
@@ -831,6 +920,10 @@ ConditionGeo::get_geo_int(const sockaddr *addr) const
   int64_t ret = -1;
   int v       = 4;
 
+  if (!addr) {
+    return 0;
+  }
+
   switch (_geo_qual) {
   // Country Databse
   case GEO_QUAL_COUNTRY_ISO:
@@ -857,7 +950,7 @@ ConditionGeo::get_geo_int(const sockaddr *addr) const
     break;
 
   case GEO_QUAL_ASN: {
-    const char *asn_name = NULL;
+    const char *asn_name = nullptr;
 
     switch (addr->sa_family) {
     case AF_INET:
@@ -881,7 +974,7 @@ ConditionGeo::get_geo_int(const sockaddr *addr) const
       while (*asn_name && !(isdigit(*asn_name))) {
         ++asn_name;
       }
-      ret = strtol(asn_name, NULL, 10);
+      ret = strtol(asn_name, nullptr, 10);
     }
   }
     TSDebug(PLUGIN_NAME, "eval(): Client IPv%d seems to come from ASN #: %" PRId64, v, ret);
@@ -903,7 +996,7 @@ const char *
 ConditionGeo::get_geo_string(const sockaddr *addr) const
 {
   TSError("[%s] No Geo library available!", PLUGIN_NAME);
-  return NULL;
+  return nullptr;
 }
 
 int64_t
@@ -923,7 +1016,7 @@ ConditionGeo::initialize(Parser &p)
   if (is_int_type()) {
     Matchers<int64_t> *match = new Matchers<int64_t>(_cond_op);
 
-    match->set(static_cast<int64_t>(strtol(p.get_arg().c_str(), NULL, 10)));
+    match->set(static_cast<int64_t>(strtol(p.get_arg().c_str(), nullptr, 10)));
     _matcher = match;
   } else {
     // The default is to have a string matcher
@@ -1004,7 +1097,7 @@ ConditionId::initialize(Parser &p)
   if (_id_qual == ID_QUAL_REQUEST) {
     Matchers<uint64_t> *match = new Matchers<uint64_t>(_cond_op);
 
-    match->set(static_cast<uint64_t>(strtol(p.get_arg().c_str(), NULL, 10)));
+    match->set(static_cast<uint64_t>(strtol(p.get_arg().c_str(), nullptr, 10)));
     _matcher = match;
   } else {
     // The default is to have a string matcher
@@ -1051,12 +1144,10 @@ ConditionId::append_value(std::string &s, const Resources &res ATS_UNUSED)
     }
   } break;
   case ID_QUAL_UNIQUE: {
-    std::ostringstream oss;
-    TSUuid process = TSProcessUuidGet();
+    char uuid[TS_CRUUID_STRING_LEN + 1];
 
-    if (process) {
-      oss << TSUuidStringGet(process) << '-' << TSHttpTxnIdGet(res.txnp);
-      s += oss.str();
+    if (TS_SUCCESS == TSClientRequestUuidGet(res.txnp, uuid)) {
+      s += uuid;
     }
   } break;
   }
@@ -1081,3 +1172,378 @@ ConditionId::eval(const Resources &res)
     return rval;
   }
 }
+
+void
+ConditionCidr::initialize(Parser &p)
+{
+  Condition::initialize(p);
+
+  MatcherType *match = new MatcherType(_cond_op);
+
+  match->set(p.get_arg());
+  _matcher = match;
+}
+
+void
+ConditionCidr::set_qualifier(const std::string &q)
+{
+  bool ok = true;
+  int cidr;
+  char *endp;
+
+  Condition::set_qualifier(q);
+
+  TSDebug(PLUGIN_NAME, "\tParsing %%{CIDR:%s} qualifier", q.c_str());
+  cidr = strtol(q.c_str(), &endp, 10);
+  if (cidr >= 0 && cidr <= 32) {
+    _v4_mask.s_addr = UINT32_MAX >> (32 - cidr);
+    _v4_cidr        = cidr;
+    if (endp && (*endp == ',' || *endp == '/' || *endp == ':')) {
+      cidr = strtol(endp + 1, nullptr, 10);
+      if (cidr >= 0 && cidr <= 128) {
+        _v6_cidr = cidr;
+      } else {
+        TSError("[%s] Bad CIDR mask for IPv6: %s", PLUGIN_NAME, q.c_str());
+        ok = false;
+      }
+    }
+  } else {
+    TSError("[%s] Bad CIDR mask for IPv4: %s", PLUGIN_NAME, q.c_str());
+    ok = false;
+  }
+
+  // Update the bit-masks
+  if (ok) {
+    _create_masks();
+  }
+}
+
+bool
+ConditionCidr::eval(const Resources &res)
+{
+  std::string s;
+
+  append_value(s, res);
+  TSDebug(PLUGIN_NAME, "Evaluating CIDR()");
+
+  return static_cast<MatcherType *>(_matcher)->test(s);
+}
+
+void
+ConditionCidr::append_value(std::string &s, const Resources &res)
+{
+  struct sockaddr const *addr = TSHttpTxnClientAddrGet(res.txnp);
+
+  if (addr) {
+    switch (addr->sa_family) {
+    case AF_INET: {
+      char res[INET_ADDRSTRLEN];
+      struct in_addr ipv4 = reinterpret_cast<const struct sockaddr_in *>(addr)->sin_addr;
+
+      ipv4.s_addr &= _v4_mask.s_addr;
+      inet_ntop(AF_INET, &ipv4, res, INET_ADDRSTRLEN);
+      if (res[0]) {
+        s += res;
+      }
+    } break;
+    case AF_INET6: {
+      char res[INET6_ADDRSTRLEN];
+      struct in6_addr ipv6 = reinterpret_cast<const struct sockaddr_in6 *>(addr)->sin6_addr;
+
+      if (_v6_zero_bytes > 0) {
+        memset(&ipv6.s6_addr[16 - _v6_zero_bytes], 0, _v6_zero_bytes);
+      }
+      if (_v6_mask != 0xff) {
+        ipv6.s6_addr[16 - _v6_zero_bytes] &= _v6_mask;
+      }
+      inet_ntop(AF_INET6, &ipv6, res, INET6_ADDRSTRLEN);
+      if (res[0]) {
+        s += res;
+      }
+    } break;
+    }
+  } else {
+    s += "0.0.0.0"; // No client addr for some reason ...
+  }
+}
+
+// Little helper function, to create the masks
+void
+ConditionCidr::_create_masks()
+{
+  _v4_mask.s_addr = htonl(UINT32_MAX << (32 - _v4_cidr));
+  _v6_zero_bytes  = (128 - _v6_cidr) / 8;
+  _v6_mask        = 0xff >> ((128 - _v6_cidr) % 8);
+}
+
+void
+ConditionInbound::initialize(Parser &p)
+{
+  Condition::initialize(p);
+
+  MatcherType *match = new MatcherType(_cond_op);
+
+  match->set(p.get_arg());
+  _matcher = match;
+}
+
+void
+ConditionInbound::set_qualifier(const std::string &q)
+{
+  Condition::set_qualifier(q);
+
+  TSDebug(PLUGIN_NAME, "\tParsing %%{%s:%s} qualifier", TAG, q.c_str());
+
+  if (q == "LOCAL-ADDR") {
+    _net_qual = NET_QUAL_LOCAL_ADDR;
+  } else if (q == "LOCAL-PORT") {
+    _net_qual = NET_QUAL_LOCAL_PORT;
+  } else if (q == "REMOTE-ADDR") {
+    _net_qual = NET_QUAL_REMOTE_ADDR;
+  } else if (q == "REMOTE-PORT") {
+    _net_qual = NET_QUAL_REMOTE_PORT;
+  } else if (q == "TLS") {
+    _net_qual = NET_QUAL_TLS;
+  } else if (q == "H2") {
+    _net_qual = NET_QUAL_H2;
+  } else if (q == "IPV4") {
+    _net_qual = NET_QUAL_IPV4;
+  } else if (q == "IPV6") {
+    _net_qual = NET_QUAL_IPV6;
+  } else if (q == "IP-FAMILY") {
+    _net_qual = NET_QUAL_IP_FAMILY;
+  } else if (q == "STACK") {
+    _net_qual = NET_QUAL_STACK;
+  } else {
+    TSError("[%s] Unknown %s() qualifier: %s", PLUGIN_NAME, TAG, q.c_str());
+  }
+}
+
+bool
+ConditionInbound::eval(const Resources &res)
+{
+  std::string s;
+
+  append_value(s, res);
+  bool rval = static_cast<const Matchers<std::string> *>(_matcher)->test(s);
+
+  TSDebug(PLUGIN_NAME, "Evaluating %s(): %s - rval: %d", TAG, s.c_str(), rval);
+
+  return rval;
+}
+
+void
+ConditionInbound::append_value(std::string &s, const Resources &res)
+{
+  this->append_value(s, res, _net_qual);
+}
+
+void
+ConditionInbound::append_value(std::string &s, const Resources &res, NetworkSessionQualifiers qual)
+{
+  const char *zret = nullptr;
+  char text[INET6_ADDRSTRLEN];
+
+  switch (qual) {
+  case NET_QUAL_LOCAL_ADDR: {
+    zret = getIP(TSHttpTxnIncomingAddrGet(res.txnp), text);
+  } break;
+  case NET_QUAL_LOCAL_PORT: {
+    uint16_t port = getPort(TSHttpTxnIncomingAddrGet(res.txnp));
+    snprintf(text, sizeof(text), "%d", port);
+    zret = text;
+  } break;
+  case NET_QUAL_REMOTE_ADDR: {
+    zret = getIP(TSHttpTxnClientAddrGet(res.txnp), text);
+  } break;
+  case NET_QUAL_REMOTE_PORT: {
+    uint16_t port = getPort(TSHttpTxnClientAddrGet(res.txnp));
+    snprintf(text, sizeof(text), "%d", port);
+    zret = text;
+  } break;
+  case NET_QUAL_TLS:
+    zret = TSHttpTxnClientProtocolStackContains(res.txnp, "tls/");
+    break;
+  case NET_QUAL_H2:
+    zret = TSHttpTxnClientProtocolStackContains(res.txnp, "h2");
+    break;
+  case NET_QUAL_IPV4:
+    zret = TSHttpTxnClientProtocolStackContains(res.txnp, "ipv4");
+    break;
+  case NET_QUAL_IPV6:
+    zret = TSHttpTxnClientProtocolStackContains(res.txnp, "ipv6");
+    break;
+  case NET_QUAL_IP_FAMILY:
+    zret = TSHttpTxnClientProtocolStackContains(res.txnp, "ip");
+    break;
+  case NET_QUAL_STACK: {
+    std::array<char const *, 8> tags;
+    int count  = 0;
+    size_t len = 0;
+    TSHttpTxnClientProtocolStackGet(res.txnp, tags.size(), tags.data(), &count);
+    for (int i = 0; i < count; ++i) {
+      len += 1 + strlen(tags[i]);
+    }
+    s.reserve(len);
+    for (int i = 0; i < count; ++i) {
+      if (i) {
+        s += ',';
+      }
+      s += tags[i];
+    }
+  } break;
+  }
+
+  if (zret) {
+    s += zret;
+  }
+}
+
+ConditionStringLiteral::ConditionStringLiteral(const std::string &v)
+{
+  TSDebug(PLUGIN_NAME_DBG, "Calling CTOR for ConditionStringLiteral");
+  _literal = v;
+}
+
+void
+ConditionStringLiteral::append_value(std::string &s, const Resources &res)
+{
+  s += _literal;
+  TSDebug(PLUGIN_NAME, "Appending '%s' to evaluation value", _literal.c_str());
+}
+
+bool
+ConditionStringLiteral::eval(const Resources &res)
+{
+  TSDebug(PLUGIN_NAME, "Evaluating StringLiteral");
+
+  return static_cast<const MatcherType *>(_matcher)->test(_literal);
+}
+
+ConditionExpandableString::ConditionExpandableString(const std::string &v)
+{
+  TSDebug(PLUGIN_NAME_DBG, "Calling CTOR for ConditionExpandableString");
+  _value = v;
+}
+
+bool
+ConditionExpandableString::eval(const Resources &res)
+{
+  std::string s;
+
+  append_value(s, res);
+
+  return static_cast<const MatcherType *>(_matcher)->test(s);
+}
+
+void
+ConditionExpandableString::append_value(std::string &s, const Resources &res)
+{
+  VariableExpander ve(_value);
+  s += ve.expand(res);
+  TSDebug(PLUGIN_NAME, "Appending to evaluation value -> %s", s.c_str());
+}
+// ConditionSessionTransactCount
+void
+ConditionSessionTransactCount::initialize(Parser &p)
+{
+  Condition::initialize(p);
+  MatcherType *match     = new MatcherType(_cond_op);
+  std::string const &arg = p.get_arg();
+
+  match->set(strtol(arg.c_str(), nullptr, 10));
+  _matcher = match;
+}
+
+bool
+ConditionSessionTransactCount::eval(const Resources &res)
+{
+  int const val = TSHttpTxnServerSsnTransactionCount(res.txnp);
+
+  TSDebug(PLUGIN_NAME, "Evaluating SSN-TXN-COUNT()");
+  return static_cast<MatcherType *>(_matcher)->test(val);
+}
+
+void
+ConditionSessionTransactCount::append_value(std::string &s, Resources const &res)
+{
+  char value[32]; // enough for UINT64_MAX
+  int const count  = TSHttpTxnServerSsnTransactionCount(res.txnp);
+  int const length = ink_fast_itoa(count, value, sizeof(value));
+
+  if (length > 0) {
+    TSDebug(PLUGIN_NAME, "Appending SSN-TXN-COUNT %s to evaluation value %.*s", _qualifier.c_str(), length, value);
+    s.append(value, length);
+  }
+}
+
+void
+ConditionTcpInfo::initialize(Parser &p)
+{
+  Condition::initialize(p);
+  TSDebug(PLUGIN_NAME, "Initializing TCP Info");
+  MatcherType *match     = new MatcherType(_cond_op);
+  std::string const &arg = p.get_arg();
+
+  match->set(strtol(arg.c_str(), nullptr, 10));
+  _matcher = match;
+}
+
+void
+ConditionTcpInfo::initialize_hooks()
+{
+  add_allowed_hook(TS_HTTP_TXN_START_HOOK);
+  add_allowed_hook(TS_HTTP_TXN_CLOSE_HOOK);
+  add_allowed_hook(TS_HTTP_SEND_RESPONSE_HDR_HOOK);
+}
+
+bool
+ConditionTcpInfo::eval(const Resources &res)
+{
+  std::string s;
+
+  append_value(s, res);
+  bool rval = static_cast<const Matchers<std::string> *>(_matcher)->test(s);
+
+  TSDebug(PLUGIN_NAME, "Evaluating TCP-Info: %s - rval: %d", s.c_str(), rval);
+
+  return rval;
+}
+
+void
+ConditionTcpInfo::append_value(std::string &s, Resources const &res)
+{
+#if defined(TCP_INFO) && defined(HAVE_STRUCT_TCP_INFO)
+  if (TSHttpTxnIsInternal(res.txnp)) {
+    TSDebug(PLUGIN_NAME, "No TCP-INFO available for internal transactions");
+    return;
+  }
+  TSReturnCode tsSsn;
+  int fd;
+  struct tcp_info info;
+  socklen_t tcp_info_len = sizeof(info);
+  tsSsn                  = TSHttpTxnClientFdGet(res.txnp, &fd);
+  if (tsSsn != TS_SUCCESS || fd <= 0) {
+    TSDebug(PLUGIN_NAME, "error getting the client socket fd from ssn");
+  }
+  if (getsockopt(fd, IPPROTO_TCP, TCP_INFO, &info, &tcp_info_len) != 0) {
+    TSDebug(PLUGIN_NAME, "getsockopt(%d, TCP_INFO) failed: %s", fd, strerror(errno));
+  }
+
+  if (tsSsn == TS_SUCCESS) {
+    if (tcp_info_len > 0) {
+      char buf[12 * 4 + 9]; // 4x uint32's + 4x "; " + '\0'
+#if !defined(freebsd) || defined(__GLIBC__)
+      snprintf(buf, sizeof(buf), "%" PRIu32 ";%" PRIu32 ";%" PRIu32 ";%" PRIu32 "", info.tcpi_rtt, info.tcpi_rto,
+               info.tcpi_snd_cwnd, info.tcpi_retrans);
+#else
+      snprintf(buf, sizeof(buf), "%" PRIu32 ";%" PRIu32 ";%" PRIu32 ";%" PRIu32 "", info.tcpi_rtt, info.tcpi_rto,
+               info.tcpi_snd_cwnd, info.__tcpi_retrans);
+#endif
+      s += buf;
+    }
+  }
+#else
+  s += "-";
+#endif
+}
